library app_control;

import 'package:flutter/material.dart';
import 'datalayer/data_factory.dart';

import 'datalayer/entity/updatedetailsweb.dart';
import 'datalayer/entity/update_details_req_web.dart';
import 'datalayer/entity/response_web.dart';

import 'package:ingdao/ingdao.dart';
import 'package:ingdao/dataflow/control/dfquery.dart';
import 'package:ingdao/dataflow/control/dfconverter.dart';
import 'package:ingdao/dataflow/control/dfresponse.dart';
import 'package:ingdao/dataflow/control/dfqueryentityjoin.dart';
import 'package:ingdao/dataflow/repository/dfbaserepository.dart';
import 'package:ingdao/dataflow/entity/dfbaseentity.dart';
import 'package:ingdao/dataflow/field/dfbasefield.dart';
import 'package:ingdao/util/util.dart';
import 'app_update_page.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:object_mapper/object_mapper.dart';

enum AppControlVersionUpdateResponseType {
  updateIsNotNeeded,
  userChooseUpdateNow,
  userChooseUpdateLater,
  userChooseNotToUpdateLastTime
}

extension AppControlVersionUpdateResponseTypeExtension on AppControlVersionUpdateResponseType {

  int get value {
    switch (this) {
      case AppControlVersionUpdateResponseType.updateIsNotNeeded:
        return 0;
      case AppControlVersionUpdateResponseType.userChooseUpdateNow:
        return 1;
      case AppControlVersionUpdateResponseType.userChooseUpdateLater:
        return 2;
      case AppControlVersionUpdateResponseType.userChooseNotToUpdateLastTime:
        return 3;                        
      default:
        return null;
    }
  }
}

class AppControl {

  static bool dataEncrypt = false;
  static String serverUrl;
  static String endPoint;
  static String dbName = "appcontrol";
  static String accessToken;

  static Future<void> checkVersionUpdate(BuildContext context, bool alwaysAskForUpdate, Function callback) async {
    DfFactory dbFactory = await DataFactory.instance.getManipulator(RepositoryType.web);
    DfBaseRepository repository = dbFactory.currentRepository;
    UpdateDetailsReqWeb req = repository.model.entities[0];
    //UpdateDetailsField updateDetailsField = UpdateDetailsField.createFieldBasedOnRepo(repository);

    ExtraPackageInfo extraPackageInfo = await Util.getExtraPackageInfo();
    DfExtraReq extraReq = DfExtraReq();
    req.appName = extraPackageInfo.packageInfo.appName;
    req.platform = extraPackageInfo.platformType.value;
    req.version = extraPackageInfo.packageInfo.version;
    extraReq.entity = req;

    DfQuery query = DfQuery(purpose: DfQueryPurpose.ADD, 
                            queryFields: [], 
                            entityJoin: DfQueryEntityJoin(entities: [req], 
                            joins: []), //JoinType.innerJoin 
                            //queryStr can be something like "(g and g) or g"  - g represent a DfQueryCriteriaGroup object 
                            criteriaGroups: null, //DfQueryCriteriaGroups(queryGroups: [DfQueryCriteriaGroup(criterias: criterias, andOrType: AndOrType.and)], queryStr: ""), 
                            orderBy: null, 
                            paging: null,
                            extraReq: extraReq);

    List<DfResponse> resps = await dbFactory.save([query]);

    if (resps[0].error != null) {
      print("Got Error during version checking: ${resps[0].error.code} ${resps[0].error.message}");
    }

    

    ResponseWeb result = resps[0].renderResultToEntity();
    UpdateDetailsWeb details = result.data;
    print("Created on: ${details.createdOn} ${DfConverter.dateToString(details.createdOn)}");
    if (details.updateLevel == AppUpdateLevelType.updated.value) {
      callback(AppControlVersionUpdateResponseType.updateIsNotNeeded);
    }
    else {
      bool canRoute = true;
      SharedPreferences prefs = await SharedPreferences.getInstance();
      const String TAG_LASTCHECKAPPVERSION = "lastCheckAppVersion";
      if (details.updateLevel == AppUpdateLevelType.optionalUpdate.value) {
        String lastCheckAppVersion = prefs.getString(TAG_LASTCHECKAPPVERSION);
        if (lastCheckAppVersion == details.version && !alwaysAskForUpdate) {
          canRoute = false;
        }
      }

      if (canRoute) {
        Util.routeTo(context, AppUpdatePage(
          updateDetails: details,
          callback: (action) {
            if (action == AppControlVersionUpdateResponseType.userChooseUpdateLater) {
              prefs.setString(TAG_LASTCHECKAPPVERSION, details.version);
            }
            callback(action);
          },
        ));
      }
      else {
        callback(AppControlVersionUpdateResponseType.userChooseNotToUpdateLastTime);
      }
    }
  }

}
