import 'package:flutter/material.dart';
import 'datalayer/entity/updatedetailsweb.dart';
import 'bloc/slideup_panel_bloc.dart';
import 'bloc/bloc_provider.dart';
import 'package:ingdao/util/just_animate.dart';
import 'package:ingdao/util/util.dart';
import 'package:url_launcher/url_launcher.dart';
import 'app_control.dart';

class SlideUpPanel extends StatelessWidget {

  final UpdateDetailsWeb updateDetails;
  final Function callback;

  SlideUpPanel({Key key, this.updateDetails, this.callback}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<SlideupPanelBlocData>(
      stream: BlocProvider.of<SlideupPanelBloc>(context).stream,
      builder: (context, snapshot) {
        bool _panelShow = false;
        bool _butCloseShow = true;
        String _error;
        if (snapshot.connectionState != ConnectionState.waiting) {
          _panelShow = snapshot.data.showPanel;
          _butCloseShow = snapshot.data.showButClose;
          _error = snapshot.data.error;
        }

        //work and return on UI part
        return !_panelShow ? Container() : Container(
          height: MediaQuery.of(context).size.height - 400,
          child: Stack(
            clipBehavior: Clip.none, //overflow then just visible.
            children: [
              Container(
                decoration: BoxDecoration(
                  color: HexColor(updateDetails.slideUpPanelColor),
                  borderRadius: BorderRadius.only(
                    topLeft:  Radius.circular(20),
                    topRight: Radius.circular(20)
                  )
                ),
                child: Padding(
                  padding: EdgeInsets.all(20),
                  child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(updateDetails.updateTitle, style: TextStyle(color: HexColor(updateDetails.updateTitleColor), fontSize: updateDetails.updateTitleSize, fontWeight: FontWeight.bold)),
                      SizedBox(height: 8),
                      Text(updateDetails.updateMessage, style: TextStyle(color: HexColor(updateDetails.updateMessageColor), fontSize: updateDetails.updateMessageSize, fontWeight: FontWeight.bold)),                  
                      SizedBox(height: 30),
                      RaisedButton(
                        shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(18.0),
                        side: BorderSide(color: HexColor(updateDetails.butProceedColor))),
                        onPressed: () {
                          callback(AppControlVersionUpdateResponseType.userChooseUpdateNow);
                        },
                        color: HexColor(updateDetails.butProceedColor),
                        textColor: Colors.white,
                        child: Text(updateDetails.butProceedText,
                        style: TextStyle(fontSize: 14)),
                      ),
                      updateDetails.updateLevel == AppUpdateLevelType.compulsoryUpdate.value || !_butCloseShow ? Container() : RaisedButton(
                        shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(18.0),
                        side: BorderSide(color: HexColor(updateDetails.butLaterColor))),
                        onPressed: () {
                          callback(AppControlVersionUpdateResponseType.userChooseUpdateLater);
                        },
                        color: HexColor(updateDetails.butLaterColor),
                        textColor: Colors.white,
                        child: Text(updateDetails.butLaterText,
                        style: TextStyle(fontSize: 14)),
                      ),
                      SizedBox(height: 50),
                      _error == null ? Container() : Text(_error, style: TextStyle(fontSize: 12, color: Colors.red))
                    ],
                  )
                )
              ),
              Positioned(
                top: -30,
                right: 30,
                child: updateDetails.updateLevel == AppUpdateLevelType.compulsoryUpdate.value || !_butCloseShow ? Container() : FloatingActionButton(
                  child: Icon(Icons.close, color: Colors.white),
                  backgroundColor: Colors.pinkAccent, 
                  onPressed: () {
                    callback(AppControlVersionUpdateResponseType.userChooseUpdateLater);
                  },
                )
              )
            ],
          )
        );

      }
    );
  }

}

class AppUpdatePage extends JustAnimateWidget {

  final UpdateDetailsWeb updateDetails;
  final Function callback;
  SlideupPanelBloc _bloc = SlideupPanelBloc();

  AppUpdatePage({Key key, this.updateDetails, this.callback});

  @override
  void customInitState() {
    Util.doLater(afterMilliSec: 300, doFor: () => animate());
    //animate();
  }

  @override
  Widget getChild(BuildContext context) {
    return BlocProvider<SlideupPanelBloc>(
      bloc: _bloc,
      child: Scaffold(
        backgroundColor: Colors.transparent,
        body: Container(
          decoration: BoxDecoration(
            gradient: new LinearGradient(
                    begin: Alignment.topCenter,
                    end: Alignment.bottomCenter,
                    colors: [
                      HexColor(updateDetails.backgroundColorTop),
                      HexColor(updateDetails.backgroundColorBottom)
                    ],
                  )),
          child: Stack(
            alignment: Alignment.bottomCenter,

            children: [
              Container(
              child: Column(
                children: [
                  Container()
                ],
              ),
            ),
            Positioned(
              top: MediaQuery.of(context).size.height * updateDetails.topMarginRatio,
              left: 0,
              right: 0,
              child: Container(
                child: Center(
                  child: Column(
                    children: [
                      Image.network(updateDetails.logoUrl, fit: BoxFit.contain, width: updateDetails.logoWidth, height: updateDetails.logoHeight),
                      SizedBox(height: 8),
                      Text(updateDetails.appTitle, style: TextStyle(color: HexColor(updateDetails.appTitleColor) , fontSize: updateDetails.appTitleSize, fontWeight: FontWeight.bold))
                    ],
                  )
                )
              )
            ),
            createSlideAnimationWidget(SlideUpPanel(updateDetails: updateDetails, callback: (callbackAction) async {
              if (callbackAction == AppControlVersionUpdateResponseType.userChooseUpdateLater) {               
                _bloc.setButCloseShow(false);
                animate();
              }
              else {
                if (await canLaunch(updateDetails.updateUrl)) {
                  await launch(updateDetails.updateUrl);
                } else {
                  _bloc.setError(updateDetails.updateUrlFailLaunchMessage);
                }
              }

              callback(callbackAction);
            }))
            ],
          )  
        ),
      )
    );
  }

  @override
  void doWhenAnimationBegin(BuildContext context) {
    print("Animation Start");
    _bloc.togglePanelShow();
  }

  @override
  void doWhenAnimationEnd(BuildContext context) {
    print("Animation End");
    _bloc.togglePanelShow();
    Navigator.of(context).pop();
  }

  @override
  bool isEnableHardwareBackButton() {
    return false;
  }
}