import 'package:ingdao/dataflow/control/dfquery.dart';
import 'package:ingdao/dataflow/control/dfqueryorderby.dart';
import 'package:ingdao/dataflow/entity/dfbaseentity.dart';
import 'package:ingdao/dataflow/entity/dfwebentity.dart';
import 'package:ingdao/dataflow/field/dfbasefield.dart';
import 'package:ingdao/dataflow/field/dfwebfield.dart';
import '../../field/tanktimesheet/tanktimesheet_field.dart';
import 'package:ingdao/dataflow/field/dffieldtype.dart';
import 'package:object_mapper/object_mapper.dart';
import '../../../appsetting.dart';

class TankTimesheetWeb extends DfWebEntity with TankTimesheetField {

  String id;
  String tankId;
  String name;
  DateTime markOn;
  bool active;
  //Tank tank;
  //List<TsNote> notes;

  TankTimesheetWeb ({String pId, String pTankId, String pName, DateTime pMarkOn, bool pActive}) : super(contextPath: "tanktimesheet") {
    fId = DfWebField(entity: this, name: "id", fieldType: DfFieldType.text(), getObjectValue: () => id, setObjectValue: (v) => id = v, value: pId);
    fTankId = DfWebField(entity: this, name: "tankId", fieldType: DfFieldType.text(), getObjectValue: () => tankId, setObjectValue: (v) => tankId = v, value: pTankId);
    fName = DfWebField(entity: this, name: "name", fieldType: DfFieldType.text(), getObjectValue: () => name, setObjectValue: (v) => name = v, value: pName);
    fMarkOn = DfWebField(entity: this, name: "markOn", fieldType: DfFieldType.datetime(), getObjectValue: () => markOn, setObjectValue: (v) => markOn = v, value: pMarkOn);
    fActive = DfWebField(entity: this, name: "active", fieldType: DfFieldType.boolean(), getObjectValue: () => active, setObjectValue: (v) => active = v, value: pActive);
    //fTank = DfWebField(entity: this, name: "tank", fieldType: DfFieldType.custom(), value: null);
    //fNotes = DfWebField(entity: this, name: "notes", fieldType: DfFieldType.custom(), value: null);
  }

  //Define what are the fields will map back the result from http response Map<String,dynamic>
  @override
  List<DfWebField> fields() {
     return [fId,fTankId,fName,fMarkOn,fActive];
  }

  @override
  DfWebReqParam convertToWebReqParam(DfQuery query) {
    DfWebReqParam param = DfWebReqParam();
    param.setAuthToken(AppSetting.token);

    if (query.purpose == DfQueryPurpose.FETCH) {

      if (query.paging != null) {
        param.reqParams["pageNumber"] = "${query.paging.pageNumber}";
        param.reqParams["perPageCount"] = "${query.paging.perPageCount}";
      }
        
      if (query.orderBy != null) {
        int i = 0;
        for (var orderBy in query.orderBy) {
          ++i;
          param.reqParams["orderBy$i"] = orderBy.field.name;
          param.reqParams["asc$i"] = orderBy.orderByType ==OrderByType.asc ? "true" : "false";
        }
      }

        if (query.criteriaGroups != null) {
          query.criteriaGroups.queryGroups.forEach((grp) => grp.criterias.forEach((cri) => param.pathVariables.add(cri.values[0])) );
        }

        return param;
    }
    return null;
  }

  @override
  DfBaseEntity<DfBaseField> newInstance() {
    return TankTimesheetWeb();
  }

  @override
  DfBaseEntity<DfWebField> parseExecRawJsonResultToEntity(DfQueryPurpose purpose, Mapper map) {
    if (purpose == DfQueryPurpose.FETCH)
      return map.toObject<TankTimesheetWeb>();

    return null;
  }

  @override
  void customFieldJsonMap(Mapper map, List<DfWebField> fields) {
    for (var field in fields) {
      // if (field == fData) {
      //   map<UpdateDetailsWeb>(field.name, field.getObjectValue(), (v) => field.setObjectValue(v), field.getTransform());        
      // }
    }
  }
}