import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import '../entity/fetchentityresult.dart';

class InfiniteScrollList extends StatefulWidget {
  final String title;
  final Function dataProvider;
  final Function cellDisplayWidget;
  final int perPageCount;
  final String searchName;
  InfiniteScrollList({Key key, this.title, this.dataProvider, this.cellDisplayWidget, this.perPageCount = 10, this.searchName}) : super(key:key);

  @override
  _InfiniteScrollListState createState() => _InfiniteScrollListState();
}

class _InfiniteScrollListState extends State<InfiniteScrollList> {
  bool _hasMore;
  int _pageNumber;
  bool _error;
  String _errorMsg;
  bool _loading;
  List _listData;
  final int _nextPageThreshold = 5;
  String _internalSearchName;

  @override
  void initState() {
    super.initState();
    _hasMore = true;
    _pageNumber = 1;
    _error = false;
    _errorMsg = null;
    _loading = true;
    _listData = [];
    fetchingList();
  }

  Widget _errorMessagePanel() {
    return Padding(padding: const EdgeInsets.all(16), child: Text("Error: $_errorMsg while loading, tap to try again."));
  }

  @override
  Widget build(BuildContext context) {
    // return Scaffold(
    //   appBar: AppBar(title: Text(widget.title)),
    //   body: getBody()
    // );
    return Column(children: [Text(triggerResetListData(widget.searchName)), Expanded(child: getBody())]);
  }

  String triggerResetListData(String searchName) {
    print("invoke: triggerResetListData");

    if (_internalSearchName == null && (widget.searchName == null || widget.searchName.length == 0)) {
      print("no need triggerResetListData - first time empty");
    }
    else if (_internalSearchName != widget.searchName) {
      _internalSearchName = widget.searchName;
      print("Reset by invoking fetchingList");
      _hasMore = true;
      _pageNumber = 1;
      _error = false;
      _errorMsg = null;
      _loading = true;
      _listData = [];
      fetchingList();
    }
    else {
      print("no need triggerResetListData - current input same as previous");
    }
    return "";
  }  

  Widget getBody() {
    if (_listData.isEmpty) {
      if (_loading) {
        return Center(child: Padding(padding: const EdgeInsets.all(8), child: CircularProgressIndicator()));
      }
      else if (_error) {
        return Center(
          child: InkWell(
            onTap: () {
              setState(() {
                _loading = true;
                _error = false;
                fetchingList();
              });
            },
            child: _errorMessagePanel(),
          )
        );
      }
    } else {
      return ListView.builder(
        itemCount: _listData.length + (_hasMore ? 1 : 0),
        itemBuilder: (context, index) {
          if (index == _listData.length - _nextPageThreshold) {
            fetchingList();            
          }
          if (index == _listData.length) {
            if (_error) {
              return Center(
                child: InkWell(
                  onTap: () {
                    setState(() {
                      _loading = true;
                      _error = false;
                      fetchingList();
                    });
                  },
                  child: _errorMessagePanel(),
                )
              );
            } else {
              return Center(child: _errorMessagePanel());
            }
          } //end if (index == _users.length)
          return widget.cellDisplayWidget(_listData[index]);
        });
    }
    return Container();
  }

  Future<void> fetchingList() async {
    try {
      FetchEntityResult result = await widget.dataProvider(widget.searchName, _pageNumber, widget.perPageCount);
      //print("fetchedListData: ${fetchedListData.length}");
      if (result.error != null) {
        setState(() {
          _loading = false;
          _error = true;
          _errorMsg = result.error;
        });
      }
      else {
        setState(() {
          var fetchedListData = result.entities;
          _hasMore = fetchedListData.length == widget.perPageCount;
          _loading = false;
          _pageNumber = _pageNumber + 1;
          _listData.addAll(fetchedListData);
        });
      }      
    } catch (e) {
      setState(() {
        _loading = false;
        _error = true;
        _errorMsg = e.toString();
      });
    }
  }
}