import 'dart:async';

import 'package:flutter/material.dart';
import 'infinitescrolllist.dart';

import '../service/applservice.dart';
import '../entity/userfield.dart';

class UserListScr extends StatefulWidget {
  final AppService appService;
  UserListScr({Key key, this.appService}) : super(key:key);

  @override
  _UserListScrState createState() => _UserListScrState();
}

class _UserListScrState extends State<UserListScr>  {

  final _searchQueryController = new TextEditingController();
  Timer _debounce;
  String _searchBarQuery = "";
  int _debouncetime = 1500;

  @override
  void initState() {
    _searchBarQuery = "";
    _searchQueryController.addListener(_onSearchChanged);
    super.initState();
  }

  @override
  void dispose() {
    _searchQueryController.removeListener(_onSearchChanged);
    _searchQueryController.dispose();
    super.dispose();
  }

  _onSearchChanged() {
    if (_debounce?.isActive ?? false) _debounce.cancel();
    _debounce = Timer(Duration(milliseconds: _debouncetime), () {
      //if (_searchQueryController.text != "") {
        ///here you perform your search
        setState(() {
          _searchBarQuery = _searchQueryController.text;
          print("searchBarQuery: $_searchBarQuery");
        });
      //}
    });
  }

  Widget formSearchBar() {
    return Padding(padding: const EdgeInsets.fromLTRB(3, 16, 3, 0) , child: TextField(
          controller: _searchQueryController,
          decoration: new InputDecoration(
              border: new OutlineInputBorder(
                borderRadius: const BorderRadius.all(
                  const Radius.circular(5.0),
                ),
              ),
              filled: true,
              hintStyle: new TextStyle(color: Colors.grey[800]),
              labelText: "Search",
              fillColor: Colors.white70),
        )
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: "Data layer - User Management",
      theme: ThemeData(
        primarySwatch: Colors.teal,
      ),
      home: Scaffold(
        appBar: AppBar(title: Text("Data layer - User Management"),
          actions: [
            IconButton(icon: Icon(Icons.add), onPressed: () {

            })
          ],
        ),
        body: Container(
          child: Column(children: [
            formSearchBar(),
            Expanded(
              child: InfiniteScrollList(searchName: _searchBarQuery, dataProvider: (String listSearchName, int pageNumber, int limit) {
                print("Pass in pageNumber: $pageNumber and limit: $limit");
                return widget.appService.fetchUser(listSearchName, pageNumber, limit);
              }, cellDisplayWidget: (UserField user) {
                  return InkWell(child: Card(child: ListTile(
                                      leading: CircleAvatar(
                                                backgroundImage: NetworkImage(user.fPicture.value)
                                                ),
                                      title: Text(user.fName.value),
                                      subtitle: Text(user.fEmail.value),
                              )),
                              onTap: () {
                                print("Prepare to edit");
                              })
                          ;

              }, perPageCount: 20,)
            )             
          ],)
        ) 
        
       
      )
    );
  }
}