import 'dart:convert';
import 'package:intl/intl.dart';
import 'package:object_mapper/object_mapper.dart';

class DfConverter {
  static String dtTimeFormat;
  static const DT_TIME_FORMAT = 'yyyy-MM-dd HH:mm:ss';
  static const ARR_SEPARATOR = '@@@';

  static String arrToString(List list, {String sep = ARR_SEPARATOR}) {
    if (list == null) return null;
    String result = "";
    list.forEach((item) {
      if (result.length > 0)
        result = "$result$sep";
      result = "$result${item.toString()}";
    });
    return result;
  }

  static int boolToInt(bool val) {
    return val ? 1 : 0;
  }

  static bool intToBool(int val) {
    return val == 1;
  }

  static List<String> stringToList(String str, {String sep = ARR_SEPARATOR}) {
    if (str == null) return null;
    return str.split(sep);
  }

  static String dateToString(DateTime dt, {String format = DT_TIME_FORMAT}) {
    //final DateTime now = DateTime.now();
    String finalFormat = dtTimeFormat != null ? dtTimeFormat : format;

    int tIdx = finalFormat.indexOf('T');

    finalFormat = finalFormat.replaceAll('T', ' ');
    final DateFormat formatter = DateFormat(finalFormat);
    String dtStr = formatter.format(dt);
    if (tIdx != null && tIdx >= 0)
      dtStr = dtStr.replaceRange(tIdx, tIdx+1, 'T');
    return dtStr;
  }

  static DateTime stringToDate(String dtStr, {String format = DT_TIME_FORMAT}) {
    //final DateTime now = DateTime.now();
    String finalFormat = dtTimeFormat != null ? dtTimeFormat : format;
    finalFormat = finalFormat.replaceAll('T', ' ');

    final DateFormat formatter = DateFormat(finalFormat);
    final str = dtStr.replaceAll('T', ' ');
    //print("Receive date str: $dtStr format to ${formatter.parse(str)}");
    return formatter.parse(str);
  }

  static String dictToJsonString(Map<String,dynamic> dict) {
    if (dict == null) return null;
    return json.encode(dict);
  }

  static String arrDictToJsonString(List<Map<String,dynamic>> dict) {
    if (dict == null) return null;
    return json.encode(dict);
  }

  static Map<String,dynamic> jsonStringToDict(String str) {
    List<Map<String,dynamic>> arr = jsonStringToDictList(str);
    return arr != null && arr.length > 0 ? arr[0] : null;
  }

  static List<Map<String,dynamic>> jsonStringToDictList(String str) {
    if (str == null) return null;
    List<Map<String,dynamic>> result = List<Map<String,dynamic>>();
    if (str != null) {
      String tStr = str.trim();
      if (tStr.startsWith("[")) {
        List data = jsonDecode(tStr);
        for (Map<String,dynamic> item in data) {
          result.add(item);
        }
      }
      else {
        result.add(jsonDecode(tStr));
      }
    }
    return result;
  }
}

class StrDateTimeTransform implements Transformable<DateTime, String> {
  String pattern;
  StrDateTimeTransform({this.pattern = DfConverter.DT_TIME_FORMAT});

  //
  @override
  DateTime fromJson(value) {
    try {

      if (value == null) return null;
      if (value is String) return DfConverter.stringToDate(value);
      if (value is int) return DateTime.fromMillisecondsSinceEpoch(DateUnit.seconds.addScale(value.toDouble()).toInt());
      if (value < 0) return null;
      return null;
    } catch (e) {
      print("Error during parse date: $e for value: $value");
      return null;
    }
  }

  @override
  String toJson(DateTime value) {
    if (value == null) return null;

    return DfConverter.dateToString(value);
  }
}

class IntBoolTransform implements Transformable<bool, int> {
  IntBoolTransform();

  //
  @override
  bool fromJson(value) {
    try {
      //print("IntBoolTransform raw value: $value");
      if (value == null) return null;
      if (value is bool) return value;
      if (value is int) {
        //print("IntBoolTransform value: $value");
        return DfConverter.intToBool(value);
      }
      if (value < 0) return null;
      return null;
    } catch (e) {
      return null;
    }
  }

  @override
  int toJson(bool value) {
    if (value == null) return null;

    return DfConverter.boolToInt(value);
  }
}