import 'package:meta/meta.dart';
import '../field/dfbasefield.dart';
import 'dfconverter.dart';

enum OperType {
  eq,
  neq,
  isNull,
  isNotNull,
  between,
  gt,
  lt,
  egt,
  elt,
  inClause,
  like
}

extension OperTypeExtension on OperType {

  String get name {
    switch (this) {
      case OperType.eq:
        return '=';
      case OperType.like:
        return 'like';        
      case OperType.neq:
        return '<>';
     case OperType.isNull:
        return 'IS NULL';
     case OperType.isNotNull:
        return 'IS NOT NULL';
     case OperType.between:
        return 'BETWEEN';
     case OperType.gt:
        return '>';
     case OperType.lt:
        return '<';
     case OperType.egt:
        return '>=';
     case OperType.elt:
        return '<=';
     case OperType.inClause:
        return 'in';                                                   
      default:
        return null;
    }
  }

  String desp() {
    return this.name;
  }
}

class DfQueryCriteria {
  DfBaseField field;
  OperType oper;
  List values;

  DfQueryCriteria({@required this.field, @required this.oper, @required this.values}) {
    if (values != null && values.length > 0) {
      List newValues = List();
      values.forEach((val) {
        if (val is DateTime) {
          newValues.add(DfConverter.dateToString(val));
        }
        else if (val is bool) {
          newValues.add(DfConverter.boolToInt(val));
        }
        else {
          newValues.add(val);          
        }
      });
    }
  }

  String toCriteriaStr() {
    String fieldName = field.name;
    String questionMark = "";
    if (oper == OperType.inClause) {
      for (int i = 0; i < values.length; i++) {
        if (questionMark.length > 0) {
          questionMark = "$questionMark, ";
        }
        questionMark += '?';
      }
      questionMark = '(' + questionMark + ')';
    }
    else if (oper == OperType.between) {
      questionMark = "? and ?";
    }
    else if (oper == OperType.isNotNull || oper == OperType.isNull) {
      questionMark = "";
    }
    else {
            questionMark = "?";
    }
    String operName = oper.name;
    if (oper == OperType.like) {
      return "lower(${field.entity.entityName}.$fieldName) $operName $questionMark";
    }
    return "${field.entity.entityName}.$fieldName $operName $questionMark";
  }  
}