import 'package:meta/meta.dart';
import 'dfbaseentity.dart';
import '../field/dfwebfield.dart';
import '../field/dfbasefield.dart';
import '../control/dfquery.dart';
import '../field/dffieldtype.dart';
import 'package:object_mapper/object_mapper.dart';

enum HttpMethodType {
  GET,
  POST,
  PUT,
  PATCH,
  DELETE
}

extension FieldTypeExtension on HttpMethodType {

  String get name {
    switch (this) {
      case HttpMethodType.GET:
        return 'GET';
      case HttpMethodType.POST:
        return 'POST';
     case HttpMethodType.PUT:
        return 'PUT';
     case HttpMethodType.PATCH:
        return 'PATCH';
     case HttpMethodType.DELETE:
        return 'DELETE';        
      default:
        return null;
    }
  }

  String desp() {
    return this.name;
  }
}

abstract class DfWebEntity extends DfBaseEntity<DfWebField> {

  Map<String,String> headers;

  DfWebEntity({@required String contextPath, this.headers}) : super(entityName: contextPath);

  @override
  void doYourJsonMap(Mapper map, List<DfWebField> fields) { 
    //print("call db doYourJsonMap");
    List<DfWebField> customFields = [];   
    for (int i = 0; i < fields.length; i++) {
      if (fields[i].fieldType.type != FieldType.custom && fields[i].getObjectValue != null && fields[i].setObjectValue != null) {
        map(fields[i].name, fields[i].getObjectValue(), (v) => fields[i].setObjectValue(v), fields[i].getTransform());
      }
      else if (fields[i].getObjectValue != null && fields[i].setObjectValue != null) {
        customFields.add(fields[i]);
      }
    }
    customFieldJsonMap(map, customFields);
  }

  String formUrl(DfQuery query) {
    DfWebReqParam params = convertToWebReqParam(query);  
    String url = entityName;
    if (params.pathVariables != null) {
      params.pathVariables.forEach((pVar) => url = "$url/$pVar");
    }
    return url;
  }

  DfBaseField getSelectQueryCountField() {
    return DfWebField(entity: this, fieldType: DfFieldType.integer(), name: "count(*)");
  }

  DfWebReqParam convertToWebReqParam(DfQuery query);  
}

class DfWebReqParam {
  List<String> pathVariables;
  Map<String,String> reqParams;
  Map<String,dynamic> reqBody;
  Map<String,String> _headers;
  Map<String,String> get headers => _headers;

  DfWebReqParam({String authToken, Map<String,String> headers, this.pathVariables, this.reqParams, this.reqBody}) {
    if (pathVariables == null) {
      pathVariables = List<String>();
    }
    if (reqParams == null) {
      reqParams = Map<String,String>();
    }
    if (reqBody == null) {
      reqBody = Map<String,dynamic>();
    }
    if (headers == null) {
      _headers = Map<String,String>();
    }
    setAuthToken(authToken);
  }

  void setAuthToken(String token) {
    if (token != null)
      _headers["Authorization"] = "Bearer $token";
  }
}