import 'dfvalidator.dart';
import '../control/dfresponse.dart';

class MinDateValidateException extends DfError {
   MinDateValidateException(DateTime dtTime) : super(code: 10013, message: "Input date/time is less than ${dtTime.toString()}");
}

class MaxDateValidateException extends DfError {
   MaxDateValidateException(DateTime dtTime) : super(code: 10014, message: "Input date/time is greater than ${dtTime.toString()}");
}

class LtNowValidateException extends DfError {
   LtNowValidateException() : super(code: 10015, message: "Input date/time must be less than now.");
}

class DfDateTimeValidator extends DfValidator {

  DateTime minDtTime;
  DateTime maxDtTime;
  bool isLtNowCheck = false;

  DfDateTimeValidator({bool isAllowEmpty = true, this.minDtTime, this.maxDtTime, this.isLtNowCheck = false}) : super(isAllowEmpty: isAllowEmpty);

  LtNowValidateException ltNowValidate() {
    if (emptyValidate() == null && isLtNowCheck && field.value != null) {
      DateTime val = field.value as DateTime;
      return val.compareTo(DateTime.now()) >= 0  ? LtNowValidateException() : null;
    }
    return null;
  }

  MinDateValidateException minDateTimeValidate() {
    if (emptyValidate() == null && minDtTime != null && field.value != null) {
      DateTime val = field.value as DateTime;
      return val.compareTo(minDtTime) < 0  ? MinDateValidateException(minDtTime) : null;
    }
    return null;
  }

  MaxDateValidateException maxDateTimeValidate() {
    if (emptyValidate() == null && maxDtTime != null && field.value != null) {
      DateTime val = field.value as DateTime;
      return val.compareTo(maxDtTime) > 0 ? MaxDateValidateException(maxDtTime) : null;
    }
    return null;
  }

  List<DfError> validate() {
    List<DfError> errors = List<DfError>();
    DfError emptyErr = emptyValidate();
    if (emptyErr != null) {
      errors.add(emptyErr);
    }

    DfError minValueErr = minDateTimeValidate();
    if (minValueErr != null) {
      errors.add(minValueErr);
    }

    DfError maxLenErr = maxDateTimeValidate();
    if (maxLenErr != null) {
      errors.add(maxLenErr);
    }

    DfError ltNowErr = ltNowValidate();
    if (ltNowErr != null) {
      errors.add(ltNowErr);
    }

    return errors.length > 0 ? errors : null;
  }
}