import 'dfvalidator.dart';
import 'dfstringvalidator.dart';
import 'dfnumericvalidator.dart';
import 'dfdatetimevalidator.dart';

enum FieldType {
  text,
  integer,
  real,
  blob,
  custom
}

extension FieldTypeExtension on FieldType {

  String get name {
    switch (this) {
      case FieldType.text:
        return 'TEXT';
      case FieldType.integer:
        return 'INTEGER';
     case FieldType.real:
        return 'REAL';
     case FieldType.blob:
        return 'BLOB';
     case FieldType.custom:
        return 'CUSTOM';                                        
      default:
        return null;
    }
  }

  String desp() {
    return this.name;
  }
}

class DfFieldType {
  FieldType type;
  DfValidator validator;
  bool isDateTime = false;
  bool isBoolean = false;

  DfFieldType.text({DfStringValidator strValidator}) {
    type = FieldType.text;
    validator = strValidator;
    isDateTime = false;
    isBoolean = false;
  }

    DfFieldType.custom() {
    type = FieldType.custom;
    isDateTime = false;
    isBoolean = false;
  }

  DfFieldType.integer({DfNumericValidator numericValidator}) {
    type = FieldType.integer;
    validator = numericValidator;
    isDateTime = false;
    isBoolean = false;
  }

  DfFieldType.real({DfNumericValidator numericValidator}) {
    type = FieldType.real;
    validator = numericValidator;
    isDateTime = false;
    isBoolean = false;
  }

  DfFieldType.blob({DfValidator baseValidator}) {
    type = FieldType.blob;
    validator = baseValidator;
    isDateTime = false;
    isBoolean = false;
  }

  DfFieldType.datetime({DfDateTimeValidator dateTimeValidator}) {
    type = FieldType.text;
    validator = dateTimeValidator;
    isDateTime = true;
    isBoolean = false;
  }

  DfFieldType.boolean({DfValidator baseValidator}) {
    type = FieldType.integer;
    validator = baseValidator;
    isDateTime = false;
    isBoolean = true;
  }
}