import 'package:flutter/material.dart';

enum JustAnimateType {
  slideTransition
}

abstract class JustAnimateWidget extends StatefulWidget {

  AnimationController _animationController;
  Animation<Offset> _offsetAnimation;

  Widget getChild(BuildContext context);
  void doWhenAnimationBegin(BuildContext context);
  void doWhenAnimationEnd(BuildContext context);

  SlideTransition createSlideAnimationWidget(Widget widget) {
    return SlideTransition(
      position: _offsetAnimation,
      child: widget
    );
  }

  bool isEnableHardwareBackButton();

  void animate() {
    if (_animationController != null) {
      if (_animationController.isDismissed) {
        _animationController.forward();
      }
      else {
        _animationController.reverse();        
      }
    }
  }

  void customInitState();

  @override
  _JustAnimateState createState() => _JustAnimateState();

}

class _JustAnimateState extends State<JustAnimateWidget> with SingleTickerProviderStateMixin {

  @override
  void initState() {
    super.initState();
    widget._animationController = AnimationController(
      duration: const Duration(milliseconds: 600),
      vsync: this
    );
    widget._offsetAnimation = Tween<Offset>(
                        begin: const Offset(0, 1),
                        end: Offset.zero
                      ).animate(CurvedAnimation(
                          parent: widget._animationController, 
                          curve: Curves.decelerate)
                        )..addStatusListener((status) {
                          if (status == AnimationStatus.forward) {
                            widget.doWhenAnimationBegin(context);
                          }
                          else if (status == AnimationStatus.dismissed) {
                            widget.doWhenAnimationEnd(context);                        
                          }
                        });
    widget.customInitState();                        
  }

  @override
  void dispose() {
    super.dispose();
    widget._animationController.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return new WillPopScope(
      onWillPop: () async => widget.isEnableHardwareBackButton(),
      child: widget.getChild(context)
    );
  } 
}