import 'package:flutter/material.dart';
import 'dart:io' show Platform;
import 'package:package_info/package_info.dart';

enum PlatformType {
  android,
  iphone,
  ipad
}

extension PlatformTypeExtension on PlatformType {

  int get value {
    switch (this) {
      case PlatformType.android:
        return 1;
      case PlatformType.iphone:
        return 2;
      case PlatformType.ipad:
        return 3;                
      default:
        return null;
    }
  }
}

class ExtraPackageInfo {
  PackageInfo packageInfo;
  PlatformType platformType;

  ExtraPackageInfo({this.packageInfo, this.platformType});
}

class HexColor extends Color {
  static int _getColorFromHex(String hexColor) {
    hexColor = hexColor.toUpperCase().replaceAll("#", "");
    if (hexColor.length == 6) {
      hexColor = "FF" + hexColor;
    }
    return int.parse(hexColor, radix: 16);
  }

  HexColor(final String hexColor) : super(_getColorFromHex(hexColor));
}

class Util {

  static Future<ExtraPackageInfo> getExtraPackageInfo() async {
    ExtraPackageInfo info = ExtraPackageInfo();
    if (Platform.isAndroid) {
      info.platformType = PlatformType.android;
    } else {
      final data = MediaQueryData.fromWindow(WidgetsBinding.instance.window);
      info.platformType = data.size.shortestSide < 600 ? PlatformType.iphone : PlatformType.ipad;
    }
    info.packageInfo = await PackageInfo.fromPlatform();
    return info;
  } 

  static void doLater({int afterMilliSec, Function doFor}) {
    Future.delayed(Duration(milliseconds: afterMilliSec), doFor);
  }

  static void routeTo(BuildContext context, Widget page, {bool replace = false}) {
    if (replace) {
      Navigator.pushReplacement(
        context,
        MaterialPageRoute(builder: (context) => page),
      );
    }
    else {
      Navigator.push(
        context,
        MaterialPageRoute(builder: (context) => page),
      );      
    }
  }

}